@@ -11,20 +11,29 @@ sub new {
     my ($klass, %args) = @_;
     
     # setup before instantiation
-    my $listen_sock;
     if (defined $ENV{SERVER_STARTER_PORT}) {
-        my ($hostport, $fd) = %{Server::Starter::server_ports()};
-        if ($hostport =~ /(.*):(\d+)/) {
-            $args{host} = $1;
-            $args{port} = $2;
-        } else {
-            $args{port} = $hostport;
+        $args{listens} = [];
+        my $server_ports = Server::Starter::server_ports();
+        for my $hostport (keys %$server_ports) {
+            my $fd = $server_ports->{$hostport};
+            my $listen = {};
+            if ($hostport =~ /(.*):(\d+)/) {
+                $listen->{host} = $1;
+                $listen->{port} = $2;
+            } else {
+                $listen->{port} = $hostport;
+            }
+            $listen->{sock} = IO::Socket::INET->new(
+                Proto => 'tcp',
+            ) or die "failed to create socket:$!";
+            $listen->{sock}->fdopen($fd, 'w')
+                or die "failed to bind to listening socket:$!";
+            unless (@{$args{listens}}) {
+                $args{host} = $listen->{host};
+                $args{port} = $listen->{port};
+            }
+            $args{listens}[$fd] = $listen;
         }
-        $listen_sock = IO::Socket::INET->new(
-            Proto => 'tcp',
-        ) or die "failed to create socket:$!";
-        $listen_sock->fdopen($fd, 'w')
-            or die "failed to bind to listening socket:$!";
     }
     my $max_workers = 10;
     for (qw(max_workers workers)) {
@@ -35,8 +44,6 @@ sub new {
     # instantiate and set the variables
     my $self = $klass->SUPER::new(%args);
     $self->{is_multiprocess} = 1;
-    $self->{listen_sock} = $listen_sock
-        if $listen_sock;
     $self->{max_workers} = $max_workers;
     
     $self;
