@@ -13,6 +13,8 @@ use Plack::Util;
 use Plack::TempBuffer;
 use POSIX qw(EINTR EAGAIN EWOULDBLOCK);
 use Socket qw(IPPROTO_TCP TCP_NODELAY);
+use File::Temp qw(tempfile);
+use Fcntl qw(:flock);
 
 use Try::Tiny;
 use Time::HiRes qw(time);
@@ -85,7 +87,8 @@ sub setup_listener {
         };
     }
 
-    for my $listen (grep {defined $_} @{$self->{listens}}) {
+    my @listens = grep {defined $_} @{$self->{listens}};
+    for my $listen (@listens) {
         my $family = Socket::sockaddr_family(getsockname($listen->{sock}));
         $listen->{_is_tcp} = $family != AF_UNIX;
 
@@ -96,6 +99,17 @@ sub setup_listener {
         }
     }
 
+    if (scalar(@listens) > 1) {
+        $self->{lock_path} ||= do {
+            my ($fh, $lock_path) = tempfile(UNLINK => 1);
+            # closing the file handle explicitly for two reasons
+            # 1) tempfile retains the handle when UNLINK is set
+            # 2) tempfile implicitely locks the file on OS X
+            close $fh;
+            $lock_path;
+        };
+    }
+
     $self->{server_ready}->($self);
 }
 
@@ -121,7 +135,7 @@ sub accept_loop {
     while (! defined $max_reqs_per_child || $proc_req_count < $max_reqs_per_child) {
         my ($conn, $peer, $listen) = $acceptor->();
         $self->{_is_deferred_accept} = $listen->{_using_defer_accept};
-        $conn->blocking(0)
+        defined($conn->blocking(0))
             or die "failed to set socket to nonblocking mode:$!";
         my ($peerport, $peerhost, $peeraddr) = (0, undef, undef);
         if ($listen->{_is_tcp}) {
@@ -194,13 +208,22 @@ sub _get_acceptor {
         my @fds;
         my $rin = '';
         for my $listen (@listens) {
-            $listen->{sock}->blocking(0);
+            defined($listen->{sock}->blocking(0))
+	        or die "failed to set listening socket to non-blocking mode:$!";
             my $fd = fileno($listen->{sock});
             push @fds, $fd;
             vec($rin, $fd, 1) = 1;
         }
+
+        open(my $lock_fh, '>', $self->{lock_path})
+            or die "failed to open lock file:@{[$self->{lock_path}]}:$!";
+
         return sub {
             while (1) {
+                while (! flock($lock_fh, LOCK_EX)) {
+                    next if $! == EINTR;
+                    die "failed to lock file:@{[$self->{lock_path}]}:$!";
+                }
                 my $nfound = select(my $rout = $rin, undef, undef, undef);
                 for (my $i = 0; $nfound > 0; ++$i) {
                     my $fd = $fds[$i];
@@ -208,9 +231,11 @@ sub _get_acceptor {
                     --$nfound;
                     my $listen = $self->{listens}[$fd];
                     if (my ($conn, $peer) = $listen->{sock}->accept) {
-                        return ($conn, $peer, $listen)
+                        flock($lock_fh, LOCK_UN);
+                        return ($conn, $peer, $listen);
                     }
                 }
+                flock($lock_fh, LOCK_UN);
             }
         };
     }
